#include "objective_flags.hpp"
#include "utils/godot_macros.hpp"

void ObjectiveData::_bind_methods() {
#define CLASSNAME ObjectiveData
	GDSIGNAL("progress_changed");
	GDSIGNAL("is_done");
	GDPROPERTY(target_count, gd::Variant::INT);
	GDFUNCTION(get_is_completed);
}

bool ObjectiveData::get_is_completed() const {
	for(gd::Ref<ObjectiveData> sub_objective : this->sub_objectives) {
		if(!sub_objective->get_is_completed()) {
			return false;
		}
	}
	return this->completed_count >= this->target_count;
}

void ObjectiveData::set_target_count(int value) {
	this->target_count = value;
}

int ObjectiveData::get_target_count() const {
	return this->target_count;
}

void ObjectiveData::set_completed_count(int value) {
	this->completed_count = gd::MIN(value, this->target_count);
	this->emit_signal("progress_changed");
}

int ObjectiveData::get_completed_count() const {
	return this->completed_count;
}

void ObjectiveData::set_sub_objectives_setting(gd::Array array) {
	this->sub_objectives.clear();
	for(int i{0}; i < array.size(); ++i) {
		this->sub_objectives.push_back(array[i]);
	}
}

gd::Array ObjectiveData::get_sub_objectives_setting() const {
	gd::Array array{};
	for(gd::Ref<ObjectiveData> data : this->sub_objectives) {
		array.push_back(data);
	}
	return array;
}

void ObjectiveData::check_is_done() {
	if(this->get_is_completed()) {
		this->emit_signal("is_done");
	}
}

#undef CLASSNAME

void ObjectiveFlags::_bind_methods() {
#define CLASSNAME ObjectiveFlags
	GDSIGNAL("progress_changed", gd::PropertyInfo(gd::Variant::OBJECT, "objective", gd::PROPERTY_HINT_RESOURCE_TYPE, "ObjectiveData"));
}

void ObjectiveFlags::register_objective(gd::Ref<ObjectiveData> objective_name) {
	this->currently_active_objectives.insert(objective_name);
}

void ObjectiveFlags::set_currently_active_objectives(gd::Array array) {
	for(int i{0}; i < array.size(); ++i) {
		this->register_objective(array[i]);
	}
}
